#include "TArtNEBULAParameters.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtCore.h"
#include "TArtStoreManager.h"
#include "TArtMap.h"

#include <TDOMParser.h>
#include <TXMLNode.h>

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <vector>

//__________________________________________________________
TArtNEBULAParameters::TArtNEBULAParameters(const char* name, const char* title)
  : TNamed(name, title)
{
  TArtCore::Info(__FILE__,"Creating NEBULA setup...");

  pmap = new TArtNEBULAPlaParaMap;

  fStoreManager = TArtStoreManager::Instance();
  fStoreManager->AddParameters(this);
}

//__________________________________________________________
TArtNEBULAParameters::~TArtNEBULAParameters() 
{
  TArtNEBULAPlaParaMap::iterator itr = pmap->begin();
  std::vector<const TArtNEBULAPlaPara*> pvec;
  while(itr != pmap->end()){
    std::vector<const TArtNEBULAPlaPara*>::iterator it =
      find(pvec.begin(), pvec.end(), itr->second);
    if(it == pvec.end()){
      pvec.push_back(itr->second);
      delete itr->second;
    }
    itr->second = 0;
    ++itr;
  }
}

//__________________________________________________________
Bool_t TArtNEBULAParameters::LoadParameter(const char *xmlfile)
{
  TArtCore::Info(__FILE__,"Load parameter from %s", xmlfile);
  TDOMParser* domParser = new TDOMParser();
  domParser->SetValidate(false);
  Int_t parsecode = domParser->ParseFile(xmlfile);
  if(parsecode < 0){
    cerr << domParser->GetParseCodeMessage(parsecode) << endl;
    return false;
  }
  TXMLNode* node = domParser->GetXMLDocument()->GetRootNode();
  ParseParaList(node->GetChildren());
  return true;
}

//__________________________________________________________
void TArtNEBULAParameters::ParseParaList(TXMLNode *node)
{
  for(; node; node = node->GetNextNode()){
    if(node->GetNodeType() == TXMLNode::kXMLElementNode){ // Element Node
      if(strcmp(node->GetNodeName(), "NEBULA") == 0){
	ParseNEBULAPlaPara(node->GetChildren());
      }
    }
  }
  return;
}

//__________________________________________________________
TArtNEBULAPlaPara *TArtNEBULAParameters::ParseNEBULAPlaPara(TXMLNode *node)
{
  TArtNEBULAPlaPara* para = new TArtNEBULAPlaPara;
  Int_t tdc_geo = -1, tdc_ch = -1, qdc_geo = -1, qdc_ch = -1;

  for(; node; node = node->GetNextNode()){
    if(node->GetNodeType() == TXMLNode::kXMLElementNode){ // Element Node
      if(strcmp(node->GetNodeName(), "ID") == 0){ 
	Int_t id = atoi(node->GetText());
	para->SetID(id);
      }else if(strcmp(node->GetNodeName(), "UDLR") == 0){
	TString udlr = node->GetText();
	para->SetUDLR(udlr);
      }else if(strcmp(node->GetNodeName(), "NAME") == 0){
	TString name = node->GetText();
	para->SetDetectorName(name);
      }else if(strcmp(node->GetNodeName(), "FPl") == 0){
	Int_t fpl = atoi(node->GetText());
	para->SetFPl(fpl);
      }else if(strcmp(node->GetNodeName(), "Layer") == 0){
	Int_t layer = atoi(node->GetText());
	para->SetLayer(layer);
      }else if(strcmp(node->GetNodeName(), "SubLayer") == 0){
	Int_t sublayer = atoi(node->GetText());
	para->SetSubLayer(sublayer);
      }else if(strcmp(node->GetNodeName(), "PosX") == 0){
	Double_t posx = atof(node->GetText());
	para->SetPos(posx, 0);
      }else if(strcmp(node->GetNodeName(), "PosY") == 0){
	Double_t posy = atof(node->GetText());
	para->SetPos(posy, 1);
      }else if(strcmp(node->GetNodeName(), "PosZ") == 0){
	Double_t posz = atof(node->GetText());
	para->SetPos(posz, 2);
      }else if(strcmp(node->GetNodeName(), "TCal") == 0){
	Double_t tcal = atof(node->GetText());
	para->SetTCal(tcal);
      }else if(strcmp(node->GetNodeName(), "TOff") == 0){
	Double_t toff = atof(node->GetText());
	para->SetTOff(toff);
      }else if(strcmp(node->GetNodeName(), "QCal") == 0){
	Double_t qcal = atof(node->GetText());
	para->SetQCal(qcal);
      }else if(strcmp(node->GetNodeName(), "QPed") == 0){
	Double_t qped = atof(node->GetText());
	para->SetQPed(qped);
      }else if(strcmp(node->GetNodeName(), "Ignore") == 0){
	Int_t ignore = atoi(node->GetText());
	para->SetIgnore(ignore);
      }else if(strcmp(node->GetNodeName(), "tdc_geo") == 0){
	tdc_geo = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "tdc_ch") == 0){
	tdc_ch = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "qdc_geo") == 0){
	qdc_geo = atoi(node->GetText());
      }else if(strcmp(node->GetNodeName(), "qdc_ch") == 0){
	qdc_ch = atoi(node->GetText());
      }
    }
  }
  
  para->SetMapTDC(tdc_geo, tdc_ch);
  para->SetMapQDC(qdc_geo, qdc_ch);
  pmap->insert(TArtNEBULAPlaParaPair(para->GetTDCMap(), para));
  pmap->insert(TArtNEBULAPlaParaPair(para->GetQDCMap(), para));

  return para;
}

const TArtNEBULAPlaPara* TArtNEBULAParameters::FindNEBULAPlaPara(const TArtRIDFMap &rmap) const
{
  TArtNEBULAPlaParaMap::const_iterator itr = pmap->find(rmap);
  if(itr != pmap->end()) return itr->second;
  else return 0;
}

const TArtNEBULAPlaPara* TArtNEBULAParameters::FindNEBULAPlaPara(int id, TString udlr) const
{
  TArtNEBULAPlaParaMap::const_iterator itr = pmap->begin();
  while(itr != pmap->end()){
    const TArtNEBULAPlaPara* para = itr->second;
    if(id == para->GetID() && udlr == para->GetUDLR()) return para;
    ++itr;
  }

  return 0;
}

